//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/DistributionItems.h
//! @brief     Defines class DistributionItem and several subclasses
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_DISTRIBUTIONITEMS_H
#define BORNAGAIN_GUI_MODELS_DISTRIBUTIONITEMS_H

#include "GUI/Models/SessionItem.h"

class IDistribution1D;
class IRangedDistribution;
class RealLimitsItem;

class BA_CORE_API_ DistributionItem : public SessionItem {
private:
    static const QString P_NUMBER_OF_SAMPLES;
    static const QString P_SIGMA_FACTOR;
    static const QString P_IS_INITIALIZED;
    static const QString P_LIMITS;

public:
    explicit DistributionItem(const QString& name);

    virtual std::unique_ptr<IDistribution1D> createDistribution(double scale = 1.0) const = 0;
    void init_parameters(double value, const RealLimits& limits = RealLimits::limitless());
    void init_limits_group(const RealLimits& limits, double factor = 1.0);
    virtual void showMean(bool) = 0;

    size_t numberOfSamples() const;
    void setNumberOfSamples(size_t number);

    bool hasSigmaFactor() const;
    double sigmaFactor() const;
    void setSigmaFactor(double sigma_factor);

    bool hasLimits() const;
    SessionItem* limitsItem() const;
    RealLimitsItem* limits() const;

protected:
    virtual void init_distribution(double) {}
    void register_number_of_samples();
    void register_sigma_factor();
    void register_limits();
};

class BA_CORE_API_ SymmetricDistributionItem : public DistributionItem {
private:
    static const QString P_MEAN;

public:
    SymmetricDistributionItem(const QString& name, double mean);
    void showMean(bool flag) override;

    SessionItem* meanItem() const;
    double mean() const;
    void setMean(double mean);
    static bool isMeanPropertyName(const QString& name);

    virtual std::unique_ptr<IRangedDistribution> createIRangedDistribution(double scale) const = 0;
    virtual double deviation(double scale) const = 0;
};

class BA_CORE_API_ DistributionNoneItem : public SymmetricDistributionItem {

public:
    static const QString M_TYPE;

    DistributionNoneItem();

    std::unique_ptr<IDistribution1D> createDistribution(double scale = 1.0) const override;
    std::unique_ptr<IRangedDistribution> createIRangedDistribution(double scale) const override;
    double deviation(double scale) const override;
    void init_distribution(double value) override;
};

class BA_CORE_API_ DistributionGateItem : public DistributionItem {
private:
    static const QString P_MIN;
    static const QString P_MAX;

public:
    static const QString M_TYPE;

    DistributionGateItem();

    std::unique_ptr<IDistribution1D> createDistribution(double scale = 1.0) const override;
    void init_distribution(double value) override;
    void showMean(bool) override {}

    double minimum() const;
    double maximum() const;
    void setRange(double min, double max);
};

class BA_CORE_API_ DistributionLorentzItem : public SymmetricDistributionItem {
private:
    static const QString P_HWHM;

public:
    static const QString M_TYPE;

    DistributionLorentzItem();

    std::unique_ptr<IDistribution1D> createDistribution(double scale = 1.0) const override;
    std::unique_ptr<IRangedDistribution> createIRangedDistribution(double scale) const override;
    double deviation(double scale) const override;

    double hwhm() const;
    void setHWHM(double hwhm);

    void init_distribution(double value) override;
};

class BA_CORE_API_ DistributionGaussianItem : public SymmetricDistributionItem {
private:
    static const QString P_STD_DEV;

public:
    static const QString M_TYPE;

    DistributionGaussianItem();

    std::unique_ptr<IDistribution1D> createDistribution(double scale = 1.0) const override;
    std::unique_ptr<IRangedDistribution> createIRangedDistribution(double scale) const override;
    double deviation(double scale) const override;

    double standardDeviation() const;
    void setStandardDeviation(double std_dev);
    void init_distribution(double value) override;
};

class BA_CORE_API_ DistributionLogNormalItem : public DistributionItem {
private:
    static const QString P_MEDIAN;
    static const QString P_SCALE_PAR;

public:
    static const QString M_TYPE;

    DistributionLogNormalItem();

    std::unique_ptr<IDistribution1D> createDistribution(double scale = 1.0) const override;
    void init_distribution(double value) override;
    void showMean(bool flag) override;

    double median() const;
    void setMedian(const double median);

    double scaleParameter() const;
    void setScaleParameter(const double scale_par);
};

class BA_CORE_API_ DistributionCosineItem : public SymmetricDistributionItem {
private:
    static const QString P_SIGMA;

public:
    static const QString M_TYPE;

    DistributionCosineItem();

    std::unique_ptr<IDistribution1D> createDistribution(double scale = 1.0) const override;
    std::unique_ptr<IRangedDistribution> createIRangedDistribution(double scale) const override;
    double deviation(double scale) const override;
    void init_distribution(double value) override;

    double sigma() const;
    void setSigma(double sigma);
};

class BA_CORE_API_ DistributionTrapezoidItem : public DistributionItem {
private:
    static const QString P_CENTER;
    static const QString P_LEFTWIDTH;
    static const QString P_MIDDLEWIDTH;
    static const QString P_RIGHTWIDTH;

public:
    static const QString M_TYPE;

    DistributionTrapezoidItem();

    std::unique_ptr<IDistribution1D> createDistribution(double scale = 1.0) const override;
    void init_distribution(double value) override;
    void showMean(bool flag) override;

    double center() const;
    void setCenter(double center);

    double leftWidth() const;
    void setLeftWidth(double left_width);

    double middleWidth() const;
    void setMiddleWidth(double middle_width);

    double rightWidth() const;
    void setRightWidth(double right_width);
};

#endif // BORNAGAIN_GUI_MODELS_DISTRIBUTIONITEMS_H
