//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/GroupInfoCatalog.cpp
//! @brief     Implements class GroupInfoCatalog
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/GroupInfoCatalog.h"
#include "GUI/Models/BackgroundItems.h"
#include "GUI/Models/DistributionItems.h"
#include "GUI/Models/Error.h"
#include "GUI/Models/FootprintItems.h"
#include "GUI/Models/FormFactorItems.h"
#include "GUI/Models/MinimizerItem.h"
#include "GUI/Models/PointwiseAxisItem.h"
#include "GUI/Models/RealLimitsItems.h"
#include "GUI/Models/RectangularDetectorItem.h"
#include "GUI/Models/ResolutionFunctionItems.h"
#include "GUI/Models/SphericalDetectorItem.h"

GroupInfoCatalog::GroupInfoCatalog()
{
    GroupInfo info = GroupInfo("Distribution group");
    info.add(DistributionGateItem::M_TYPE, "Gate distribution");
    info.add(DistributionLorentzItem::M_TYPE, "Lorentz distribution");
    info.add(DistributionGaussianItem::M_TYPE, "Gaussian distribution");
    info.add(DistributionLogNormalItem::M_TYPE, "Log Normal distribution");
    info.add(DistributionCosineItem::M_TYPE, "Cosine distribution");
    info.add(DistributionTrapezoidItem::M_TYPE, "Trapezoid distribution");
    info.setDefaultType(DistributionGaussianItem::M_TYPE);
    addInfo(info);

    info = GroupInfo("Variance group");
    info.add("VarianceSimFunction", "Simulation value based");
    info.add("VarianceConstantFunction", "Constant");
    info.setDefaultType("VarianceSimFunction");
    addInfo(info);
}

GroupInfo GroupInfoCatalog::groupInfo(const QString& groupType) const
{
    for (auto& info : m_groups)
        if (info.groupType() == groupType)
            return info;

    throw Error("GroupInfoCatalog::groupInfo() -> Error. No such group '" + groupType + "'");
}

bool GroupInfoCatalog::containsGroup(const QString& groupType) const
{
    for (auto& info : m_groups)
        if (info.groupType() == groupType)
            return true;

    return false;
}

GroupInfo GroupInfoCatalog::formFactorGroupInfo()
{
    GroupInfo info("Form Factor");
    info.add(AnisoPyramidItem::M_TYPE, "Aniso Pyramid");
    info.add(BarGaussItem::M_TYPE, "BarGauss");
    info.add(BarLorentzItem::M_TYPE, "BarLorentz");
    info.add(BoxItem::M_TYPE, "Box");
    info.add(ConeItem::M_TYPE, "Cone");
    info.add(Cone6Item::M_TYPE, "Cone6");
    info.add(CuboctahedronItem::M_TYPE, "Cuboctahedron");
    info.add(CylinderItem::M_TYPE, "Cylinder");
    info.add(DodecahedronItem::M_TYPE, "Dodecahedron");
    info.add(EllipsoidalCylinderItem::M_TYPE, "Ellipsoidal Cylinder");
    info.add(FullSphereItem::M_TYPE, "Full Sphere");
    info.add(FullSpheroidItem::M_TYPE, "Full Spheroid");
    info.add(HemiEllipsoidItem::M_TYPE, "Hemi Ellipsoid");
    info.add(IcosahedronItem::M_TYPE, "Icosahedron");
    info.add(Prism3Item::M_TYPE, "Prism3");
    info.add(Prism6Item::M_TYPE, "Prism6");
    info.add(PyramidItem::M_TYPE, "Pyramid");
    info.add(CosineRippleBoxItem::M_TYPE, "CosineRippleBox");
    info.add(CosineRippleGaussItem::M_TYPE, "CosineRippleGauss");
    info.add(CosineRippleLorentzItem::M_TYPE, "CosineRippleLorentz");
    info.add(SawtoothRippleBoxItem::M_TYPE, "SawtoothRippleBox");
    info.add(SawtoothRippleGaussItem::M_TYPE, "SawtoothRippleGauss");
    info.add(SawtoothRippleLorentzItem::M_TYPE, "SawtoothRippleLorentz");
    info.add(TetrahedronItem::M_TYPE, "Tetrahedron");
    info.add(TruncatedCubeItem::M_TYPE, "Truncated Cube");
    info.add(TruncatedSphereItem::M_TYPE, "Truncated Sphere");
    info.add(TruncatedSpheroidItem::M_TYPE, "Truncated Spheroid");
    info.setDefaultType(CylinderItem::M_TYPE);

    return info;
}

void GroupInfoCatalog::addInfo(const GroupInfo& info)
{
    if (info.groupType().isEmpty())
        throw Error("GroupInfoCatalog::addInfo -> Error. Empty group name");

    if (containsGroup(info.groupType()))
        throw Error("GroupInfoCatalog::addInfo -> Error. Already exists '" + info.groupType()
                    + "'");

    m_groups.push_back(info);
}
