//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/InterferenceItems.cpp
//! @brief     Implements InterferenceItems's classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/InterferenceItems.h"
#include "Base/Const/Units.h"
#include "GUI/Models/FTDecayFunctionItems.h"
#include "GUI/Models/FTDistributionItems.h"
#include "GUI/Models/GroupItem.h"
#include "GUI/Models/Lattice2DItems.h"
#include "Sample/Aggregate/Interferences.h"

namespace {
const QString decay_function_tag = "Decay Function";
}

// TODO (when back compatibility will be broken  again)
// Make Interference1DLatticeItem::P_DECAY_FUNCTION and
// Interference2DLatticeItem::P_DECAY_FUNCTION rely on same constant

const QString InterferenceItem::P_POSITION_VARIANCE = "PositionVariance";

InterferenceItem::~InterferenceItem() = default;

void InterferenceItem::setPositionVariance(const double variance)
{
    setItemValue(P_POSITION_VARIANCE, variance);
}

InterferenceItem::InterferenceItem(const QString& modelType)
    : SessionGraphicsItem(modelType)
{
    addProperty(P_POSITION_VARIANCE, 0.0)
        ->setToolTip("Variance of the position in each dimension (nm^2)");
}

void InterferenceItem::putPositionVariance(IInterference* p_iff) const
{
    p_iff->setPositionVariance(getItemValue(P_POSITION_VARIANCE).toDouble());
}

// --------------------------------------------------------------------------------------------- //

const QString Interference1DLatticeItem::P_LENGTH = QString::fromStdString("Length");
const QString Interference1DLatticeItem::P_ROTATION_ANGLE = QString::fromStdString("Xi");
const QString Interference1DLatticeItem::P_DECAY_FUNCTION = decay_function_tag;

const QString Interference1DLatticeItem::M_TYPE = "Interference1DLattice";

Interference1DLatticeItem::Interference1DLatticeItem()
    : InterferenceItem(M_TYPE)
{
    setToolTip("Interference function of a 1D lattice");
    addProperty(P_LENGTH, 20.0 * Units::nm)->setToolTip("Lattice length in nanometers");
    addProperty(P_ROTATION_ANGLE, 0.0)
        ->setToolTip("Rotation of lattice with respect to x-axis of reference \n"
                     "frame (beam direction) in degrees ");

    GroupInfo info;
    info.add(FTDecayFunction1DCauchyItem::M_TYPE, "Cauchy 1D");
    info.add(FTDecayFunction1DGaussItem::M_TYPE, "Gauss 1D");
    info.add(FTDecayFunction1DTriangleItem::M_TYPE, "Triangle 1D");
    info.add(FTDecayFunction1DVoigtItem::M_TYPE, "Voigt 1D");
    info.setDefaultType(FTDecayFunction1DCauchyItem::M_TYPE);
    addGroupProperty(P_DECAY_FUNCTION, info)
        ->setToolTip("One-dimensional decay function (finite size effects)");
}

std::unique_ptr<IInterference>
Interference1DLatticeItem::createInterference() const
{
    auto result = std::make_unique<Interference1DLattice>(
        getItemValue(P_LENGTH).toDouble(),
        Units::deg2rad(getItemValue(P_ROTATION_ANGLE).toDouble()));
    auto pdfItem = dynamic_cast<FTDecayFunction1DItem*>(
        getGroupItem(Interference1DLatticeItem::P_DECAY_FUNCTION));
    result->setDecayFunction(*pdfItem->createFTDecayFunction());
    putPositionVariance(result.get());
    return std::unique_ptr<IInterference>(result.release());
}

void Interference1DLatticeItem::setLength(const double length)
{
    setItemValue(P_LENGTH, length);
}

void Interference1DLatticeItem::setRotationAngle(const double angle)
{
    setItemValue(P_ROTATION_ANGLE, angle);
}

// --------------------------------------------------------------------------------------------- //

const QString Interference2DAbstractLatticeItem::P_LATTICE_TYPE = "LatticeType";

const QString Interference2DAbstractLatticeItem::P_XI_INTEGRATION = "Integration_over_xi";

Lattice2DItem* Interference2DAbstractLatticeItem::latticeType() const
{
    return static_cast<Lattice2DItem*>(getGroupItem(P_LATTICE_TYPE));
}

bool Interference2DAbstractLatticeItem::xiIntegration() const
{
    return getItemValue(P_XI_INTEGRATION).toBool();
}

void Interference2DAbstractLatticeItem::setXiIntegration(bool xi_integration)
{
    setItemValue(P_XI_INTEGRATION, xi_integration);
}

Interference2DAbstractLatticeItem::Interference2DAbstractLatticeItem(
    const QString& modelType, bool xi_integration)
    : InterferenceItem(modelType)
{

    GroupInfo info;
    info.add(BasicLattice2DItem::M_TYPE, "Basic");
    info.add(SquareLattice2DItem::M_TYPE, "Square");
    info.add(HexagonalLattice2DItem::M_TYPE, "Hexagonal");
    info.setDefaultType(HexagonalLattice2DItem::M_TYPE);
    addGroupProperty(P_LATTICE_TYPE, info)->setToolTip("Type of lattice");

    addProperty(P_XI_INTEGRATION, xi_integration)
        ->setToolTip("Enables/disables averaging over the lattice rotation angle.");

    mapper()->setOnPropertyChange([this](const QString& name) {
        if (name == P_XI_INTEGRATION && isTag(P_LATTICE_TYPE)) {
            update_rotation_availability();
        }
    });

    mapper()->setOnChildPropertyChange([this](SessionItem* item, const QString&) {
        if (item->hasModelType<GroupItem>() && item->displayName() == P_LATTICE_TYPE) {
            update_rotation_availability();
        }
    });
}

//! Sets rotation property of the lattice enabled/disabled depending on integration flag.

void Interference2DAbstractLatticeItem::update_rotation_availability()
{
    auto p_lattice_item = latticeType();
    if (p_lattice_item) {
        auto angle_item = p_lattice_item->latticeRotationAngleItem();
        angle_item->setEnabled(!getItemValue(P_XI_INTEGRATION).toBool());
    }
}

// --------------------------------------------------------------------------------------------- //

const QString Interference2DLatticeItem::P_DECAY_FUNCTION = decay_function_tag;

const QString Interference2DLatticeItem::M_TYPE = "Interference2DLattice";

Interference2DLatticeItem::Interference2DLatticeItem()
    : Interference2DAbstractLatticeItem(M_TYPE, false)
{
    setToolTip("Interference function of a 2D lattice");

    GroupInfo info;
    info.add(FTDecayFunction2DCauchyItem::M_TYPE, "Cauchy 2D");
    info.add(FTDecayFunction2DGaussItem::M_TYPE, "Gauss 2D");
    info.add(FTDecayFunction2DVoigtItem::M_TYPE, "Voigt 2D");
    info.setDefaultType(FTDecayFunction2DCauchyItem::M_TYPE);
    addGroupProperty(P_DECAY_FUNCTION, info)
        ->setToolTip("Two-dimensional decay function (finite size effects)");
}

std::unique_ptr<IInterference>
Interference2DLatticeItem::createInterference() const
{
    Lattice2DItem* latticeItem = latticeType();
    std::unique_ptr<Interference2DLattice> result(
        new Interference2DLattice(*latticeItem->createLattice()));

    auto& pdfItem = groupItem<FTDecayFunction2DItem>(P_DECAY_FUNCTION);
    result->setDecayFunction(*pdfItem.createFTDecayFunction());
    result->setIntegrationOverXi(xiIntegration());
    putPositionVariance(result.get());

    return std::unique_ptr<IInterference>(result.release());
}

// --------------------------------------------------------------------------------------------- //

const QString Interference2DParaCrystalItem::P_DAMPING_LENGTH =
    QString::fromStdString("DampingLength");
const QString Interference2DParaCrystalItem::P_DOMAIN_SIZE1 =
    QString::fromStdString("DomainSize1");
const QString Interference2DParaCrystalItem::P_DOMAIN_SIZE2 =
    QString::fromStdString("DomainSize2");
const QString Interference2DParaCrystalItem::P_PDF1 = "PDF #1";
const QString Interference2DParaCrystalItem::P_PDF2 = "PDF #2";

const QString Interference2DParaCrystalItem::M_TYPE = "Interference2DParaCrystal";

Interference2DParaCrystalItem::Interference2DParaCrystalItem()
    : Interference2DAbstractLatticeItem(M_TYPE, true)
{
    setToolTip("Interference function of a two-dimensional paracrystal");
    latticeType()->latticeRotationAngleItem()->setEnabled(false);

    addProperty(P_DAMPING_LENGTH, 0.0)
        ->setToolTip("The damping (coherence) length of the paracrystal in nanometers");

    addProperty(P_DOMAIN_SIZE1, 20.0 * Units::micrometer)
        ->setToolTip("Size of the coherent domain along the first basis vector in nanometers");
    addProperty(P_DOMAIN_SIZE2, 20.0 * Units::micrometer)
        ->setToolTip("Size of the coherent domain along the second basis vector in nanometers");

    GroupInfo info;
    info.add(FTDistribution2DCauchyItem::M_TYPE, "Cauchy 2D");
    info.add(FTDistribution2DGaussItem::M_TYPE, "Gauss 2D");
    info.add(FTDistribution2DGateItem::M_TYPE, "Gate 2D");
    info.add(FTDistribution2DConeItem::M_TYPE, "Cone 2D");
    info.add(FTDistribution2DVoigtItem::M_TYPE, "Voigt 2D");
    info.setDefaultType(FTDistribution2DCauchyItem::M_TYPE);

    addGroupProperty(P_PDF1, info)
        ->setToolTip("Probability distribution in first lattice direction");
    addGroupProperty(P_PDF2, info)
        ->setToolTip("Probability distribution in second lattice direction");

    mapper()->setOnChildPropertyChange([this](SessionItem* item, const QString& property) {
        if (item->hasModelType<GroupItem>() && GroupItem::isItemsTagName(property) && isTag(P_PDF1)
            && isTag(P_PDF2))
            update_distribution_displaynames();
    });

    update_distribution_displaynames();
}

std::unique_ptr<IInterference>
Interference2DParaCrystalItem::createInterference() const
{
    Lattice2DItem* latticeItem = latticeType();

    std::unique_ptr<Interference2DParaCrystal> result(
        new Interference2DParaCrystal(*latticeItem->createLattice(), 0, 0, 0));

    result->setDampingLength(getItemValue(P_DAMPING_LENGTH).toDouble());
    result->setDomainSizes(getItemValue(P_DOMAIN_SIZE1).toDouble(),
                           getItemValue(P_DOMAIN_SIZE2).toDouble());
    result->setIntegrationOverXi(xiIntegration());

    auto& pdf1Item = groupItem<FTDistribution2DItem>(Interference2DParaCrystalItem::P_PDF1);
    auto& pdf2Item = groupItem<FTDistribution2DItem>(Interference2DParaCrystalItem::P_PDF2);
    result->setProbabilityDistributions(*pdf1Item.createFTDistribution(),
                                        *pdf2Item.createFTDistribution());

    putPositionVariance(result.get());
    return std::unique_ptr<IInterference>(result.release());
}

double Interference2DParaCrystalItem::dampingLength() const
{
    return getItemValue(P_DAMPING_LENGTH).toDouble();
}

void Interference2DParaCrystalItem::setDampingLength(const double damping_length)
{
    setItemValue(P_DAMPING_LENGTH, damping_length);
}

double Interference2DParaCrystalItem::domainSize1() const
{
    return getItemValue(P_DOMAIN_SIZE1).toDouble();
}

void Interference2DParaCrystalItem::setDomainSize1(const double domain_size1)
{
    setItemValue(P_DOMAIN_SIZE1, domain_size1);
}

double Interference2DParaCrystalItem::domainSize2() const
{
    return getItemValue(P_DOMAIN_SIZE2).toDouble();
}

void Interference2DParaCrystalItem::setDomainSize2(const double domain_size2)
{
    setItemValue(P_DOMAIN_SIZE2, domain_size2);
}

FTDistribution2DItem* Interference2DParaCrystalItem::pdf1() const
{
    return dynamic_cast<FTDistribution2DItem*>(getGroupItem(P_PDF1));
}

FTDistribution2DItem* Interference2DParaCrystalItem::pdf2() const
{
    return dynamic_cast<FTDistribution2DItem*>(getGroupItem(P_PDF2));
}

void Interference2DParaCrystalItem::update_distribution_displaynames()
{
    GroupItem* group1 = dynamic_cast<GroupItem*>(getItem(P_PDF1));
    GroupItem* group2 = dynamic_cast<GroupItem*>(getItem(P_PDF2));

    if (!group1 || !group2)
        return;

    SessionItem* pdf1 = group1->currentItem();
    SessionItem* pdf2 = group2->currentItem();

    if (pdf1 && pdf2) {
        if (pdf1->modelType() == pdf2->modelType()) {
            pdf1->setDisplayName(pdf1->modelType() + QString::number(0));
            pdf2->setDisplayName(pdf2->modelType() + QString::number(1));
        } else {
            pdf1->setDisplayName(pdf1->modelType());
            pdf2->setDisplayName(pdf2->modelType());
        }
    }
}

// --------------------------------------------------------------------------------------------- //

const QString InterferenceFinite2DLatticeItem::P_DOMAIN_SIZE_1 = "Domain_size_1";
const QString InterferenceFinite2DLatticeItem::P_DOMAIN_SIZE_2 = "Domain_size_2";

const QString InterferenceFinite2DLatticeItem::M_TYPE = "InterferenceFinite2DLattice";

InterferenceFinite2DLatticeItem::InterferenceFinite2DLatticeItem()
    : Interference2DAbstractLatticeItem(M_TYPE, false)
{
    setToolTip("Interference function of a finite 2D lattice");
    addProperty(P_DOMAIN_SIZE_1, 100u)->setToolTip("Domain size 1 in number of unit cells");
    addProperty(P_DOMAIN_SIZE_2, 100u)->setToolTip("Domain size 2 in number of unit cells");
}

std::unique_ptr<IInterference>
InterferenceFinite2DLatticeItem::createInterference() const
{
    Lattice2DItem* latticeItem = latticeType();
    auto size_1 = getItemValue(P_DOMAIN_SIZE_1).toUInt();
    auto size_2 = getItemValue(P_DOMAIN_SIZE_2).toUInt();
    std::unique_ptr<InterferenceFinite2DLattice> result(
        new InterferenceFinite2DLattice(*latticeItem->createLattice(), size_1, size_2));

    result->setIntegrationOverXi(xiIntegration());
    putPositionVariance(result.get());

    return std::unique_ptr<IInterference>(result.release());
}

void InterferenceFinite2DLatticeItem::setDomainSize1(const unsigned int domain_size1)
{
    setItemValue(P_DOMAIN_SIZE_1, domain_size1);
}

void InterferenceFinite2DLatticeItem::setDomainSize2(const unsigned int domain_size2)
{
    setItemValue(P_DOMAIN_SIZE_2, domain_size2);
}

// --------------------------------------------------------------------------------------------- //

const QString InterferenceHardDiskItem::P_RADIUS = QString::fromStdString("Radius");
const QString InterferenceHardDiskItem::P_DENSITY =
    QString::fromStdString("TotalParticleDensity");

const QString InterferenceHardDiskItem::M_TYPE = "InterferenceHardDisk";

InterferenceHardDiskItem::InterferenceHardDiskItem()
    : InterferenceItem(M_TYPE)
{
    setToolTip("Interference function for hard disk Percus-Yevick");
    addProperty(P_RADIUS, 5.0 * Units::nm)->setToolTip("Hard disk radius in nanometers");
    addProperty(P_DENSITY, 0.002)->setToolTip("Particle density in particles per square nanometer");
}

std::unique_ptr<IInterference>
InterferenceHardDiskItem::createInterference() const
{
    auto result = std::make_unique<InterferenceHardDisk>(
        getItemValue(P_RADIUS).toDouble(), getItemValue(P_DENSITY).toDouble());
    putPositionVariance(result.get());
    return std::unique_ptr<IInterference>(result.release());
}

void InterferenceHardDiskItem::setRadius(const double radius)
{
    setItemValue(P_RADIUS, radius);
}

double InterferenceHardDiskItem::density() const
{
    return getItemValue(P_DENSITY).toDouble();
}

void InterferenceHardDiskItem::setDensity(const double density)
{
    setItemValue(P_DENSITY, density);
}

// --------------------------------------------------------------------------------------------- //

const QString InterferenceRadialParaCrystalItem::P_PEAK_DISTANCE =
    QString::fromStdString("PeakDistance");
const QString InterferenceRadialParaCrystalItem::P_DAMPING_LENGTH =
    QString::fromStdString("DampingLength");
const QString InterferenceRadialParaCrystalItem::P_DOMAIN_SIZE =
    QString::fromStdString("DomainSize");
const QString InterferenceRadialParaCrystalItem::P_KAPPA =
    QString::fromStdString("SizeSpaceCoupling");
const QString InterferenceRadialParaCrystalItem::P_PDF = "PDF";

const QString InterferenceRadialParaCrystalItem::M_TYPE = "InterferenceRadialParaCrystal";

InterferenceRadialParaCrystalItem::InterferenceRadialParaCrystalItem()
    : InterferenceItem(M_TYPE)
{
    setToolTip("Interference function of a radial paracrystal");
    addProperty(P_PEAK_DISTANCE, 20.0 * Units::nm)
        ->setToolTip("Average distance to the next neighbor in nanometers");
    addProperty(P_DAMPING_LENGTH, 1000.0 * Units::nm)
        ->setToolTip("The damping (coherence) length of the paracrystal "
                     "in nanometers");
    addProperty(P_DOMAIN_SIZE, 0.0)
        ->setToolTip("Size of coherence domain along the lattice main axis "
                     "in nanometers");
    addProperty(P_KAPPA, 0.0)
        ->setToolTip("Size spacing coupling parameter of the Size Spacing "
                     "Correlation Approximation");

    GroupInfo info;
    info.add(FTDistribution1DCauchyItem::M_TYPE, "Cauchy 1D");
    info.add(FTDistribution1DGaussItem::M_TYPE, "Gauss 1D");
    info.add(FTDistribution1DGateItem::M_TYPE, "Gate 1D");
    info.add(FTDistribution1DTriangleItem::M_TYPE, "Triangle 1D");
    info.add(FTDistribution1DCosineItem::M_TYPE, "Cosine 1D");
    info.add(FTDistribution1DVoigtItem::M_TYPE, "Voigt 1D");
    info.setDefaultType(FTDistribution1DCauchyItem::M_TYPE);
    addGroupProperty(P_PDF, info)->setToolTip("One-dimensional probability distribution");
}

std::unique_ptr<IInterference>
InterferenceRadialParaCrystalItem::createInterference() const
{
    auto result = std::make_unique<InterferenceRadialParaCrystal>(
        getItemValue(P_PEAK_DISTANCE).toDouble(), getItemValue(P_DAMPING_LENGTH).toDouble());
    result->setDomainSize(getItemValue(P_DOMAIN_SIZE).toDouble());
    result->setKappa(getItemValue(P_KAPPA).toDouble());

    auto& pdfItem = groupItem<FTDistribution1DItem>(P_PDF);
    result->setProbabilityDistribution(*pdfItem.createFTDistribution());
    putPositionVariance(result.get());
    return std::unique_ptr<IInterference>(result.release());
}

void InterferenceRadialParaCrystalItem::setPeakDistance(const double peak_distance)
{
    setItemValue(P_PEAK_DISTANCE, peak_distance);
}

void InterferenceRadialParaCrystalItem::setDampingLength(const double damping_length)
{
    setItemValue(P_DAMPING_LENGTH, damping_length);
}

void InterferenceRadialParaCrystalItem::setDomainSize(const double domain_size)
{
    setItemValue(P_DOMAIN_SIZE, domain_size);
}

void InterferenceRadialParaCrystalItem::setKappa(const double kappa)
{
    setItemValue(P_KAPPA, kappa);
}
