//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/InterferenceItems.h
//! @brief     Defines InterferenceItems's classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_INTERFERENCEITEMS_H
#define BORNAGAIN_GUI_MODELS_INTERFERENCEITEMS_H

#include "GUI/Models/SessionGraphicsItem.h"

class FTDecayFunction1DItem;
class FTDecayFunction2DItem;
class FTDistribution1DItem;
class FTDistribution2DItem;
class IInterference;
class Lattice2DItem;

class BA_CORE_API_ InterferenceItem : public SessionGraphicsItem {
private:
    static const QString P_POSITION_VARIANCE;

public:
    virtual ~InterferenceItem();
    virtual std::unique_ptr<IInterference> createInterference() const = 0;
    void setPositionVariance(double variance);

protected:
    explicit InterferenceItem(const QString& modelType);
    void putPositionVariance(IInterference* p_iff) const;
};

class BA_CORE_API_ Interference1DLatticeItem : public InterferenceItem {
private:
    static const QString P_LENGTH;
    static const QString P_ROTATION_ANGLE;
    static const QString P_DECAY_FUNCTION;

public:
    static const QString M_TYPE;

    Interference1DLatticeItem();
    std::unique_ptr<IInterference> createInterference() const override;
    void setLength(double length);
    void setRotationAngle(double angle);
    template <typename T> T* setDecayFunctionType();
};

class BA_CORE_API_ Interference2DAbstractLatticeItem : public InterferenceItem {
private:
    static const QString P_LATTICE_TYPE;
    static const QString P_XI_INTEGRATION;

public:
    Lattice2DItem* latticeType() const;
    template <typename T> T* setLatticeType();

    bool xiIntegration() const;
    void setXiIntegration(bool xi_integration);

protected:
    explicit Interference2DAbstractLatticeItem(const QString& modelType,
                                                       bool xi_integration);
    void update_rotation_availability();
};

class BA_CORE_API_ Interference2DLatticeItem
    : public Interference2DAbstractLatticeItem {
private:
    static const QString P_DECAY_FUNCTION;

public:
    static const QString M_TYPE;

    Interference2DLatticeItem();
    std::unique_ptr<IInterference> createInterference() const override;

    template <typename T> T* setDecayFunctionType();
};

class BA_CORE_API_ Interference2DParaCrystalItem
    : public Interference2DAbstractLatticeItem {
private:
    static const QString P_DAMPING_LENGTH;
    static const QString P_DOMAIN_SIZE1;
    static const QString P_DOMAIN_SIZE2;
    static const QString P_PDF1;
    static const QString P_PDF2;

public:
    static const QString M_TYPE;

    Interference2DParaCrystalItem();
    std::unique_ptr<IInterference> createInterference() const override;

    double dampingLength() const;
    void setDampingLength(double damping_length);

    double domainSize1() const;
    void setDomainSize1(double domain_size1);
    double domainSize2() const;
    void setDomainSize2(double domain_size2);

    FTDistribution2DItem* pdf1() const;
    template <typename T> T* setPDF1Type();

    FTDistribution2DItem* pdf2() const;
    template <typename T> T* setPDF2Type();

private:
    void update_distribution_displaynames();
};

class BA_CORE_API_ InterferenceFinite2DLatticeItem
    : public Interference2DAbstractLatticeItem {
private:
    static const QString P_DOMAIN_SIZE_1;
    static const QString P_DOMAIN_SIZE_2;

public:
    static const QString M_TYPE;

    InterferenceFinite2DLatticeItem();
    std::unique_ptr<IInterference> createInterference() const override;

    void setDomainSize1(unsigned int domain_size1);
    void setDomainSize2(unsigned int domain_size2);
};

class BA_CORE_API_ InterferenceHardDiskItem : public InterferenceItem {
private:
    static const QString P_RADIUS;
    static const QString P_DENSITY;

public:
    static const QString M_TYPE;

    InterferenceHardDiskItem();
    std::unique_ptr<IInterference> createInterference() const override;

    void setRadius(double radius);

    double density() const;
    void setDensity(double density);
};

class BA_CORE_API_ InterferenceRadialParaCrystalItem : public InterferenceItem {
private:
    static const QString P_PEAK_DISTANCE;
    static const QString P_DAMPING_LENGTH;
    static const QString P_DOMAIN_SIZE;
    static const QString P_KAPPA;
    static const QString P_PDF;

public:
    static const QString M_TYPE;

    InterferenceRadialParaCrystalItem();
    std::unique_ptr<IInterference> createInterference() const override;

    void setPeakDistance(double peak_distance);

    void setDampingLength(double damping_length);

    void setDomainSize(double domain_size);

    void setKappa(double kappa);

    template <typename T> T* setPDFType();
};

template <typename T> T* Interference1DLatticeItem::setDecayFunctionType()
{
    static_assert(std::is_base_of<FTDecayFunction1DItem, T>::value,
                  "Class must be derived from FTDecayFunction1DItem");

    return setGroupPropertyType<T>(P_DECAY_FUNCTION);
}

template <typename T> T* Interference2DAbstractLatticeItem::setLatticeType()
{
    static_assert(std::is_base_of<Lattice2DItem, T>::value,
                  "Class must be derived from Lattice2DItem");

    return setGroupPropertyType<T>(P_LATTICE_TYPE);
}

template <typename T> T* Interference2DLatticeItem::setDecayFunctionType()
{
    static_assert(std::is_base_of<FTDecayFunction2DItem, T>::value,
                  "Class must be derived from FTDecayFunction2DItem");

    return setGroupPropertyType<T>(P_DECAY_FUNCTION);
}

template <typename T> T* Interference2DParaCrystalItem::setPDF1Type()
{
    static_assert(std::is_base_of<FTDistribution2DItem, T>::value,
                  "Class must be derived from FTDistribution2DItem");

    return setGroupPropertyType<T>(P_PDF1);
}

template <typename T> T* Interference2DParaCrystalItem::setPDF2Type()
{
    static_assert(std::is_base_of<FTDistribution2DItem, T>::value,
                  "Class must be derived from FTDistribution2DItem");

    return setGroupPropertyType<T>(P_PDF2);
}

template <typename T> T* InterferenceRadialParaCrystalItem::setPDFType()
{
    static_assert(std::is_base_of<FTDistribution1DItem, T>::value,
                  "Class must be derived from FTDistribution1DItem");

    return setGroupPropertyType<T>(P_PDF);
}

#endif // BORNAGAIN_GUI_MODELS_INTERFERENCEITEMS_H
