//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/MultiLayerItem.cpp
//! @brief     Implements class MultiLayerItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/MultiLayerItem.h"
#include "GUI/Models/LayerItem.h"
#include "GUI/Models/VectorItem.h"

namespace {
const QString external_field_tooltip = "External field (A/m)";
}

const QString MultiLayerItem::P_CROSS_CORR_LENGTH =
    QString::fromStdString("CrossCorrelationLength");
const QString MultiLayerItem::P_EXTERNAL_FIELD = "ExternalField";
const QString MultiLayerItem::T_LAYERS = "Layer tag";

const QString MultiLayerItem::M_TYPE = "MultiLayer";

MultiLayerItem::MultiLayerItem() : SessionGraphicsItem(M_TYPE)
{
    setToolTip("A multilayer to hold stack of layers");
    setItemName(M_TYPE);

    addProperty(P_CROSS_CORR_LENGTH, 0.0)
        ->setDecimals(5)
        .setToolTip("Cross correlation length of roughnesses \n"
                    "between interfaces in nanometers");
    addProperty<VectorItem>(P_EXTERNAL_FIELD)->setToolTip(external_field_tooltip);

    registerTag(T_LAYERS, 0, -1, {LayerItem::M_TYPE});
    setDefaultTag(T_LAYERS);

    mapper()->setOnChildrenChange([this](SessionItem*) { updateLayers(); });
}

QVector<ItemWithMaterial*> MultiLayerItem::itemsWithMaterial()
{
    QVector<ItemWithMaterial*> result;
    for (auto layer_item : getItems(T_LAYERS))
        result.append(dynamic_cast<LayerItem*>(layer_item)->itemsWithMaterial());
    return result;
}

double MultiLayerItem::crossCorrLength() const
{
    return getItemValue(P_CROSS_CORR_LENGTH).toDouble();
}

void MultiLayerItem::setCrossCorrLength(const double cross_corr_length)
{
    setItemValue(P_CROSS_CORR_LENGTH, cross_corr_length);
}

bool MultiLayerItem::isCrossCorrLengthPropertyName(const QString& name)
{
    return name == P_CROSS_CORR_LENGTH;
}

R3 MultiLayerItem::externalField() const
{
    return item<VectorItem>(P_EXTERNAL_FIELD)->getVector();
}

void MultiLayerItem::setExternalField(const R3& ext_field)
{
    item<VectorItem>(P_EXTERNAL_FIELD)->setVector(ext_field);
}

SessionItem* MultiLayerItem::externalFieldItem() const
{
    return getItem(P_EXTERNAL_FIELD);
}

QVector<LayerItem*> MultiLayerItem::layers() const
{
    return items<LayerItem>(T_LAYERS);
}

SessionItem* MultiLayerItem::layersItem() const
{
    return getItem(T_LAYERS);
}

void MultiLayerItem::updateLayers()
{
    QVector<LayerItem*> list = childrenOfType<LayerItem>();
    for (auto it = list.begin(); it != list.end(); ++it) {
        if (it == list.begin())
            (*it)->roughnessItem()->setEnabled(false);
        else
            (*it)->roughnessItem()->setEnabled(true);

        if (it == list.begin() || it == (list.end() - 1)) {
            (*it)->thicknessItem()->setEnabled(false);
            (*it)->setThickness(0.0);
        } else {
            (*it)->thicknessItem()->setEnabled(true);
        }
    }
}
