//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/DetectorEditor.cpp
//! @brief     Implements class DetectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/DetectorEditor.h"
#include "GUI/Models/InstrumentItems.h"
#include "GUI/Models/RectangularDetectorItem.h"
#include "GUI/Models/SphericalDetectorItem.h"
#include "GUI/Views/InstrumentWidgets/RectangularDetectorEditor.h"
#include "GUI/Views/InstrumentWidgets/SphericalDetectorEditor.h"

#include <QComboBox>
#include <QFormLayout>
#include <QStackedLayout>

#include <boost/polymorphic_cast.hpp>
using boost::polymorphic_downcast;


//==================================================================================================
// DetectorEditor
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

DetectorEditor::DetectorEditor(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QFormLayout* layout = new QFormLayout(this);

    m_combo = new QComboBox(this);
    m_combo->addItem("Rectangular", RectangularDetectorItem::M_TYPE);
    m_combo->addItem("Spherical", SphericalDetectorItem::M_TYPE);
    layout->addRow("Detector:", m_combo);
    connect(m_combo, qOverload<int>(&QComboBox::currentIndexChanged), this,
            &DetectorEditor::onDetectorSelected);

    m_stack = new QStackedLayout();
    layout->addRow(m_stack);

    m_editor_rectangular = new RectangularDetectorEditor(this);
    m_stack->addWidget(m_editor_rectangular);

    m_editor_spherical = new SphericalDetectorEditor(this);
    m_stack->addWidget(m_editor_spherical);
}

void DetectorEditor::setItem(Instrument2DItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        int idx = m_combo->findData(m_item->detector()->modelType());
        ASSERT(idx != -1);
        if (idx != m_combo->currentIndex())
            m_combo->setCurrentIndex(idx);
        else
            onDetectorSelected(idx); // trigger update manually
    }
}

void DetectorEditor::clearItem()
{
    setItem(nullptr);
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void DetectorEditor::onDetectorSelected(int)
{

    const QString model_type = m_combo->currentData().toString();
    DetectorItem* detector = m_item->setDetectorType(model_type);
    if (model_type == RectangularDetectorItem::M_TYPE) {
        m_editor_rectangular->setItem(polymorphic_downcast<RectangularDetectorItem*>(detector));
        m_stack->setCurrentWidget(m_editor_rectangular);
    } else {
        m_editor_spherical->setItem(polymorphic_downcast<SphericalDetectorItem*>(detector));
        m_stack->setCurrentWidget(m_editor_spherical);
    }
}
