//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/DistributionSelector.cpp
//! @brief     Implements class DistributionSelector
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/DistributionSelector.h"
#include "GUI/Models/DistributionItems.h"
#include "GUI/Models/ItemWithDistribution.h"
#include "GUI/Models/SessionModel.h"
#include "GUI/Views/InstrumentWidgets/DistributionForms.h"
#include "GUI/Views/InstrumentWidgets/EditDistributionDialog.h"
#include "GUI/Views/PropertyEditor/GroupInfoBox.h"

#include <QComboBox>
#include <QFormLayout>
#include <QStackedLayout>

#include <boost/polymorphic_cast.hpp>
using boost::polymorphic_downcast;

namespace {

// some temporary functions to be able to assign the value of a distribution to an other

void assignBasic(const DistributionItem* src, DistributionItem* dst)
{
    dst->setNumberOfSamples(src->numberOfSamples());
    if (src->hasSigmaFactor())
        dst->setSigmaFactor(src->sigmaFactor());
}

void assignSymmetric(const SymmetricDistributionItem* src, SymmetricDistributionItem* dst)
{
    assignBasic(src, dst);
    dst->setMean(src->mean());
}

void assign(const DistributionNoneItem* src, DistributionNoneItem* dst)
{
    dst->setMean(src->mean());
}

void assign(const DistributionGateItem* src, DistributionGateItem* dst)
{
    assignBasic(src, dst);
    dst->setRange(src->minimum(), src->maximum());
}

void assign(const DistributionLorentzItem* src, DistributionLorentzItem* dst)
{
    assignSymmetric(src, dst);
    dst->setHWHM(src->hwhm());
}

void assign(const DistributionGaussianItem* src, DistributionGaussianItem* dst)
{
    assignSymmetric(src, dst);
    dst->setStandardDeviation(src->standardDeviation());
}

void assign(const DistributionLogNormalItem* src, DistributionLogNormalItem* dst)
{
    assignBasic(src, dst);
    dst->setMedian(src->median());
    dst->setScaleParameter(src->scaleParameter());
}

void assign(const DistributionCosineItem* src, DistributionCosineItem* dst)
{
    assignSymmetric(src, dst);
    dst->setSigma(src->sigma());
}

void assign(const DistributionTrapezoidItem* src, DistributionTrapezoidItem* dst)
{
    assignBasic(src, dst);
    dst->setCenter(src->center());
    dst->setLeftWidth(src->leftWidth());
    dst->setMiddleWidth(src->middleWidth());
    dst->setRightWidth(src->rightWidth());
}

void assign(const ItemWithDistribution* src, ItemWithDistribution* dst)
{
    dst->setDistributionType(src->distribution()->modelType());
    if (src->distribution()->hasModelType<DistributionNoneItem>())
        assign(static_cast<const DistributionNoneItem*>(src->distribution()),
               static_cast<DistributionNoneItem*>(dst->distribution()));
    else if (src->distribution()->hasModelType<DistributionGateItem>())
        assign(static_cast<const DistributionGateItem*>(src->distribution()),
               static_cast<DistributionGateItem*>(dst->distribution()));
    else if (src->distribution()->hasModelType<DistributionCosineItem>())
        assign(static_cast<const DistributionCosineItem*>(src->distribution()),
               static_cast<DistributionCosineItem*>(dst->distribution()));
    else if (src->distribution()->hasModelType<DistributionLorentzItem>())
        assign(static_cast<const DistributionLorentzItem*>(src->distribution()),
               static_cast<DistributionLorentzItem*>(dst->distribution()));
    else if (src->distribution()->hasModelType<DistributionGaussianItem>())
        assign(static_cast<const DistributionGaussianItem*>(src->distribution()),
               static_cast<DistributionGaussianItem*>(dst->distribution()));
    else if (src->distribution()->hasModelType<DistributionLogNormalItem>())
        assign(static_cast<const DistributionLogNormalItem*>(src->distribution()),
               static_cast<DistributionLogNormalItem*>(dst->distribution()));
    else if (src->distribution()->hasModelType<DistributionCosineItem>())
        assign(static_cast<const DistributionCosineItem*>(src->distribution()),
               static_cast<DistributionCosineItem*>(dst->distribution()));
    else if (src->distribution()->hasModelType<DistributionTrapezoidItem>())
        assign(static_cast<const DistributionTrapezoidItem*>(src->distribution()),
               static_cast<DistributionTrapezoidItem*>(dst->distribution()));
    else
        ASSERT(false);
}

} // namespace

//==================================================================================================
// DistributionSelector
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

DistributionSelector::DistributionSelector(const InputFieldConfig& mean_config,
                                           Distributions distributions, QWidget* parent,
                                           Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QFormLayout* layout = new QFormLayout(this);

    m_combo = new QComboBox(this);
    layout->addRow("Distribution:", m_combo);

    m_stack = new QStackedLayout();
    layout->addRow(m_stack);

    m_combo->addItem("Cosine", DistributionCosineItem::M_TYPE);
    CosineDistributionForm* cosform = new CosineDistributionForm(mean_config, this);
    connect(cosform, &CosineDistributionForm::distributionChanged, this,
            &DistributionSelector::distributionChanged);
    m_stack->addWidget(cosform);
    m_forms.emplace(DistributionCosineItem::M_TYPE, cosform);

    if (distributions == Distributions::All) {
        m_combo->addItem("Gate", DistributionGateItem::M_TYPE);
        GateDistributionForm* gateform = new GateDistributionForm(this);
        connect(gateform, &GateDistributionForm::distributionChanged, this,
                &DistributionSelector::distributionChanged);
        m_stack->addWidget(gateform);
        m_forms.emplace(DistributionGateItem::M_TYPE, gateform);
    }

    m_combo->addItem("Gaussian", DistributionGaussianItem::M_TYPE);
    GaussDistributionForm* gaussform = new GaussDistributionForm(mean_config, this);
    connect(gaussform, &GaussDistributionForm::distributionChanged, this,
            &DistributionSelector::distributionChanged);
    m_stack->addWidget(gaussform);
    m_forms.emplace(DistributionGaussianItem::M_TYPE, gaussform);

    if (distributions == Distributions::All) {
        m_combo->addItem("Log normal", DistributionLogNormalItem::M_TYPE);
        LogNormalDistributionForm* lognormform = new LogNormalDistributionForm(this);
        connect(lognormform, &LogNormalDistributionForm::distributionChanged, this,
                &DistributionSelector::distributionChanged);
        m_stack->addWidget(lognormform);
        m_forms.emplace(DistributionLogNormalItem::M_TYPE, lognormform);
    }

    m_combo->addItem("Lorentz", DistributionLorentzItem::M_TYPE);
    LorentzDistributionForm* lorentzform = new LorentzDistributionForm(mean_config, this);
    connect(lorentzform, &LorentzDistributionForm::distributionChanged, this,
            &DistributionSelector::distributionChanged);
    m_stack->addWidget(lorentzform);
    m_forms.emplace(DistributionLorentzItem::M_TYPE, lorentzform);

    m_combo->addItem("None", DistributionNoneItem::M_TYPE);
    NoneDistributionForm* noneform = new NoneDistributionForm(mean_config, this);
    connect(noneform, &NoneDistributionForm::distributionChanged, this,
            &DistributionSelector::distributionChanged);
    m_stack->addWidget(noneform);
    m_forms.emplace(DistributionNoneItem::M_TYPE, noneform);

    if (distributions == Distributions::All) {
        m_combo->addItem("Trapezoid", DistributionTrapezoidItem::M_TYPE);
        TrapezoidDistributionForm* trapezoidform = new TrapezoidDistributionForm(this);
        connect(trapezoidform, &TrapezoidDistributionForm::distributionChanged, this,
                &DistributionSelector::distributionChanged);
        m_stack->addWidget(trapezoidform);
        m_forms.emplace(DistributionTrapezoidItem::M_TYPE, trapezoidform);
    }

    connect(m_combo, qOverload<int>(&QComboBox::currentIndexChanged), this,
            &DistributionSelector::onDistributionSelected);
}

void DistributionSelector::setItem(ItemWithDistribution* item)
{
    m_item = item;
    refresh();
}

ItemWithDistribution* DistributionSelector::item() const
{
    return m_item;
}

void DistributionSelector::clearItem()
{
    setItem(nullptr);
}

void DistributionSelector::refresh()
{
    setEnabled(m_item);
    if (m_item) {
        int idx = m_combo->findData(m_item->distribution()->modelType());
        ASSERT(idx != -1);
        if (idx != m_combo->currentIndex())
            m_combo->setCurrentIndex(idx);
        else
            onDistributionSelected(idx); // trigger update manually
    }
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void DistributionSelector::onDistributionSelected(int index)
{
    DistributionForm* form = m_forms[m_combo->itemData(index).toString()];
    form->setupDistribution(m_item);
    m_stack->setCurrentWidget(form);
    emit distributionChanged();
}

//==================================================================================================
// DistributionSelectorDecorated
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

DistributionSelectorDecorated::DistributionSelectorDecorated(const QString& title,
                                                             const InputFieldConfig& mean_config,
                                                             Distributions distributions,
                                                             QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f), m_title(title), m_meanconfig(mean_config), m_distributions(distributions)
{
    QVBoxLayout* layout = new QVBoxLayout(this);

    GroupInfoBox* gbox = new GroupInfoBox(title, this);
    QVBoxLayout* glayout = new QVBoxLayout(gbox);
    glayout->setMargin(0);
    m_selector = new DistributionSelector(m_meanconfig, m_distributions, gbox);
    glayout->addWidget(m_selector);
    layout->addWidget(gbox);

    connect(m_selector, &DistributionSelector::distributionChanged, this,
            &DistributionSelectorDecorated::distributionChanged);
    connect(gbox, &GroupInfoBox::clicked, this, &DistributionSelectorDecorated::showDialog);
}

void DistributionSelectorDecorated::setItem(ItemWithDistribution* item)
{
    m_selector->setItem(item);
}

void DistributionSelectorDecorated::clearItem()
{
    m_selector->clearItem();
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void DistributionSelectorDecorated::showDialog()
{
    ItemWithDistribution* item = m_selector->item();
    if (!item)
        return;

    SessionModel dummy("dummy");
    ItemWithDistribution* distr_item =
        dynamic_cast<ItemWithDistribution*>(dummy.copy(dynamic_cast<SessionItem*>(item)));
    EditDistributionDialog dialog(distr_item, m_title, m_meanconfig, m_distributions, this);
    if (dialog.exec() == QDialog::Accepted) {
        assign(distr_item, item);
        m_selector->refresh();
    }
}
