//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/DistributionSelectorWidget.h
//! @brief     A widget for selecting a distribution and setting its parameters
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONSELECTOR_H
#define BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONSELECTOR_H

#include <QWidget>

class QComboBox;
class QStackedLayout;

class DistributionForm;
class ItemWithDistribution;

#include <boost/numeric/interval.hpp>

#include <map>

/// configuration to control how the user can enter a value
struct InputFieldConfig {
    boost::numeric::interval<double> range; // range for the input value
    int decimals;                           // number of decimals shown in the input
    bool scientific;                        // whether to use a scientific number notation
};

/// enum used to configure which distributions should be available in the selector
enum class Distributions { All, Symmetric };

/// Widget for selecting a distribution (combo box) and input of the corresponding values
/// with respect to the given distribution (e.g. mean and deviation for gauss distribution)

class DistributionSelector : public QWidget {
    Q_OBJECT

public:
    DistributionSelector(const InputFieldConfig& mean_config, Distributions distributions,
                         QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(ItemWithDistribution* item);
    ItemWithDistribution* item() const;
    void clearItem();
    void refresh();

signals:
    void distributionChanged();

private slots:
    void onDistributionSelected(int index);

private:
    QComboBox* m_combo;
    QStackedLayout* m_stack;
    std::map<QString, DistributionForm*> m_forms;
    ItemWithDistribution* m_item;
};


/// DistributionSelector as above but contained in a GroupBox with a title and a button
/// to open the distribution dialog

class DistributionSelectorDecorated : public QWidget {
    Q_OBJECT

public:
    DistributionSelectorDecorated(const QString& title, const InputFieldConfig& mean_config,
                                  Distributions distributions, QWidget* parent = nullptr,
                                  Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(ItemWithDistribution* item);
    void clearItem();

signals:
    void distributionChanged();

private slots:
    void showDialog();

private:
    QString m_title;
    InputFieldConfig m_meanconfig;
    Distributions m_distributions;
    DistributionSelector* m_selector;
};

#endif // BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONSELECTOR_H
