//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/InstrumentSelectorWidget.cpp
//! @brief     Implements class InstrumentSelectorWidget
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/InstrumentSelectorWidget.h"
#include "GUI/Models/GroupItem.h"
#include "GUI/Models/InstrumentItems.h"
#include "GUI/Models/InstrumentModel.h"
#include "GUI/Models/ModelUtils.h"
#include "GUI/Models/PointwiseAxisItem.h"
#include <QAction>
#include <QListView>
#include <QMenu>
#include <QPushButton>
#include <QVBoxLayout>

InstrumentSelectorWidget::InstrumentSelectorWidget(InstrumentModel* model, QWidget* parent)
    : ItemSelectorWidget(parent)
{
    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);

    listView()->setViewMode(QListView::IconMode);
    listView()->setIconSize(QSize(96, 84));
    listView()->setMovement(QListView::Static);
    listView()->setMaximumWidth(200);
    listView()->setSpacing(12);

    listView()->setObjectName("listView");
    listView()->setStyleSheet(
        QString::fromUtf8("QListView#listView\n"
                          "{\n"
                          "   selection-background-color : rgb(98,100,105); \n"
                          "   selection-color: rgb(255,255,255);\n"
                          "   border: 1px solid rgb(98,100,105);\n"
                          "}\n"));

    setModel(model);
    layout()->setMargin(10);

    m_action_new_gisas = new QAction("New GISAS", this);
    m_action_new_gisas->setIcon(QIcon(":/images/shape-square-plus.svg"));
    m_action_new_gisas->setToolTip("Add new GISAS instrument with default settings");
    connect(m_action_new_gisas, &QAction::triggered, this, &InstrumentSelectorWidget::onNewGisas);

    m_action_new_offspecular = new QAction("New off-specular", this);
    m_action_new_offspecular->setIcon(QIcon(":/images/shape-square-plus.svg"));
    m_action_new_offspecular->setToolTip("Add new off-specular instrument with default settings");
    connect(m_action_new_offspecular, &QAction::triggered, this,
            &InstrumentSelectorWidget::onNewOffSpecular);

    m_action_new_specular = new QAction("New specular", this);
    m_action_new_specular->setIcon(QIcon(":/images/shape-square-plus.svg"));
    m_action_new_specular->setToolTip("Add new specular instrument with default settings");
    connect(m_action_new_specular, &QAction::triggered, this,
            &InstrumentSelectorWidget::onNewSpecular);

    m_action_new_depthprobe = new QAction("New depth probe", this);
    m_action_new_depthprobe->setIcon(QIcon(":/images/shape-square-plus.svg"));
    m_action_new_depthprobe->setToolTip("Add new depth probe instrument with default settings");
    connect(m_action_new_depthprobe, &QAction::triggered, this,
            &InstrumentSelectorWidget::onNewDepthProbe);

    m_action_remove = new QAction("Remove", this);
    m_action_remove->setIcon(QIcon(":/images/delete.svg"));
    m_action_remove->setToolTip("Remove selected instrument");
    connect(m_action_remove, &QAction::triggered, this, &InstrumentSelectorWidget::onRemove);

    m_action_copy = new QAction("Copy", this);
    m_action_copy->setIcon(QIcon(":/images/content-copy.svg"));
    m_action_copy->setToolTip("Make a copy of the selected instrument");
    connect(m_action_copy, &QAction::triggered, this, &InstrumentSelectorWidget::onCopy);

    m_action_separator = new QAction(this);
    m_action_separator->setSeparator(true);

    connect(selectionModel(), &QItemSelectionModel::selectionChanged, this,
            &InstrumentSelectorWidget::updateActions);

    connect(this, &InstrumentSelectorWidget::contextMenuRequest, this,
            &InstrumentSelectorWidget::onContextMenuRequest);
}

QSize InstrumentSelectorWidget::sizeHint() const
{
    return QSize(170, 400);
}

QSize InstrumentSelectorWidget::minimumSizeHint() const
{
    return QSize(96, 200);
}

QList<QAction*> InstrumentSelectorWidget::toolbarActions() const
{
    return {m_action_new_gisas,    m_action_new_offspecular,
            m_action_new_specular, m_action_new_depthprobe,
            m_action_separator,    m_action_remove,
            m_action_copy};
}

void InstrumentSelectorWidget::onContextMenuRequest(const QPoint& point)
{
    QMenu menu;
    menu.addAction(m_action_copy);
    menu.addAction(m_action_remove);
    menu.addAction(m_action_separator);
    menu.addAction(m_action_new_gisas);
    menu.addAction(m_action_new_offspecular);
    menu.addAction(m_action_new_specular);
    menu.addAction(m_action_new_depthprobe);
    menu.exec(point);
}

void InstrumentSelectorWidget::onNewGisas()
{
    newInstrument<GISASInstrumentItem>();
}

void InstrumentSelectorWidget::onNewOffSpecular()
{
    newInstrument<OffSpecularInstrumentItem>();
}

void InstrumentSelectorWidget::onNewSpecular()
{
    newInstrument<SpecularInstrumentItem>();
}

void InstrumentSelectorWidget::onNewDepthProbe()
{
    newInstrument<DepthProbeInstrumentItem>();
}

//! Removes currently selected instrument.
void InstrumentSelectorWidget::onRemove()
{
    QModelIndex index = selectionModel()->selectedIndexes().front();

    if (index.isValid())
        m_model->removeRows(index.row(), 1, QModelIndex());

    updateSelection();
}

//! Makes a copy of the currently selected instrument.
void InstrumentSelectorWidget::onCopy()
{
    QModelIndex index = selectionModel()->selectedIndexes().front();

    if (index.isValid()) {
        SessionItem* item = m_model->itemForIndex(index);
        QString nameOfCopy = suggestInstrumentName(item->itemName());
        InstrumentItem* copy =
            static_cast<InstrumentItem*>(m_model->insertNewItem(item->modelType()));
        const QString idBackup = copy->id(); // will be overwritten by the following
        for (auto child : item->children()) {
            auto tag = item->tagFromItem(child);
            m_model->copyItem(child, copy, tag);
        }
        copy->setId(idBackup);
        copy->setItemName(nameOfCopy);

        // copying non-uniform axis data
        if (auto instrument = dynamic_cast<SpecularInstrumentItem*>(item)) {
            auto axis_group = instrument->beamItem()->inclinationAxisGroup();

            auto donor_axis = axis_group->itemOfType<PointwiseAxisItem>();
            if (!donor_axis->containsNonXMLData())
                return;

            auto acceptor_axis = dynamic_cast<SpecularInstrumentItem*>(copy)
                                     ->beamItem()
                                     ->inclinationAxisGroup()
                                     ->itemOfType<PointwiseAxisItem>();
            acceptor_axis->init(*donor_axis->axis(), donor_axis->getUnitsLabel());
        }
        selectionModel()->select(m_model->indexOfItem(copy), QItemSelectionModel::ClearAndSelect);
    }
}

void InstrumentSelectorWidget::updateActions()
{
    bool enabled = selectionModel()->hasSelection();
    m_action_remove->setEnabled(enabled);
    m_action_copy->setEnabled(enabled);
}

QString InstrumentSelectorWidget::suggestInstrumentName(const QString& currentName)
{
    const QMap<QString, int> map_of_names = mapOfNames();

    QMap<QString, int>::const_iterator it = map_of_names.find(currentName);
    if (it == map_of_names.end())
        return currentName;
    else
        return QString("%1 (%2)").arg(currentName).arg(it.value() + 1);
}

QMap<QString, int> InstrumentSelectorWidget::mapOfNames()
{
    QMap<QString, int> result;

    for (auto& name : GUI::Model::ItemUtils::topItemNames(m_model)) {
        int ncopy(1);
        QRegExp regexp("\\((.*)\\)");
        if (regexp.indexIn(name) >= 0)
            ncopy = regexp.cap(1).toInt();
        name.replace(regexp.cap(0), "");
        name = name.trimmed();
        result[name] = ncopy;
    }

    return result;
}

template <class Instrument> void InstrumentSelectorWidget::newInstrument()
{
    Instrument* instrument = m_model->insertItem<Instrument>();
    instrument->setItemName(suggestInstrumentName(instrument->defaultName()));
    selectionModel()->select(m_model->indexOfItem(instrument), QItemSelectionModel::ClearAndSelect);
}
