//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/PolarizationAnalysisEditor.cpp
//! @brief     Implements class PolarizationAnalysisEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/PolarizationAnalysisEditor.h"
#include "GUI/Models/DetectorItems.h"
#include "GUI/Models/InstrumentItems.h"
#include "GUI/Models/VectorItem.h"
#include "GUI/Views/InstrumentWidgets/VectorEditor.h"

#include <QDoubleSpinBox>
#include <QFormLayout>
#include <QGridLayout>
#include <QGroupBox>


//==================================================================================================
// AnalyzerPropertiesEditor
//==================================================================================================

class AnalyzerPropertiesEditor : public QWidget {

public:
    AnalyzerPropertiesEditor(QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(DetectorItem* item);
    void clearItem();

private slots:
    void onEfficiencyValueChanged(double efficiency);
    void onTransmissionValueChanged(double transmission);

private:
    QDoubleSpinBox* m_efficiency_spinbox;
    QDoubleSpinBox* m_transmission_spinbox;
    DetectorItem* m_item;
};

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

AnalyzerPropertiesEditor::AnalyzerPropertiesEditor(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QVBoxLayout* layout = new QVBoxLayout(this);

    QGroupBox* gbox = new QGroupBox("Analyzer properties", this);
    layout->addWidget(gbox);
    QFormLayout* form = new QFormLayout(gbox);

    m_efficiency_spinbox = new QDoubleSpinBox(gbox);
    m_efficiency_spinbox->setToolTip("Efficiency of the polarization analysis");
    m_efficiency_spinbox->setDecimals(4);
    m_efficiency_spinbox->setRange(std::numeric_limits<double>::lowest(),
                                   std::numeric_limits<double>::max());
    form->addRow("Efficiency:", m_efficiency_spinbox);

    m_transmission_spinbox = new QDoubleSpinBox(gbox);
    m_transmission_spinbox->setToolTip("Total transmission of the polarization analysis");
    m_transmission_spinbox->setDecimals(4);
    m_transmission_spinbox->setRange(0, std::numeric_limits<double>::max());
    form->addRow("Transmission:", m_transmission_spinbox);

    connect(m_efficiency_spinbox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &AnalyzerPropertiesEditor::onEfficiencyValueChanged);
    connect(m_transmission_spinbox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &AnalyzerPropertiesEditor::onTransmissionValueChanged);
}

void AnalyzerPropertiesEditor::setItem(DetectorItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        m_efficiency_spinbox->setValue(m_item->analyzerEfficiency());
        m_transmission_spinbox->setValue(m_item->analyzerTotalTransmission());
    } else {
        m_efficiency_spinbox->setValue(0);
        m_transmission_spinbox->setValue(0);
    }
}

void AnalyzerPropertiesEditor::clearItem()
{
    setItem(nullptr);
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void AnalyzerPropertiesEditor::onEfficiencyValueChanged(double efficiency)
{
    if (m_item) {
        m_item->setAnalyzerEfficiency(efficiency);
    }
}

void AnalyzerPropertiesEditor::onTransmissionValueChanged(double transmission)
{
    if (m_item) {
        m_item->setAnalyzerTotalTransmission(transmission);
    }
}

//==================================================================================================
// PolarizationAnalysisEditor
//==================================================================================================

PolarizationAnalysisEditor::PolarizationAnalysisEditor(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QGridLayout* grid = new QGridLayout(this);

    m_polarizer_editor =
        new VectorEditor("Polarizer (Bloch vector)",
                         "Polarization of the beam, given as the Bloch vector (%1)", this);
    grid->addWidget(m_polarizer_editor, 0, 0);

    m_analyzer_editor = new VectorEditor("Analyzer orientation",
                                         "Direction of the polarization analysis (%1)", this);
    grid->addWidget(m_analyzer_editor, 0, 1);

    m_properties_editor = new AnalyzerPropertiesEditor(this);
    grid->addWidget(m_properties_editor, 0, 2);
}

void PolarizationAnalysisEditor::setItem(Instrument2DItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        m_polarizer_editor->setItem(m_item->beamItem()->polarizationItem());
        m_analyzer_editor->setItem(m_item->detector()->analyzerDirectionItem());
        m_properties_editor->setItem(m_item->detector());
    } else {
        m_polarizer_editor->clearItem();
        m_analyzer_editor->clearItem();
        m_properties_editor->clearItem();
    }
}
