//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/RectangularDetectorEditor.cpp
//! @brief     Implements class RectangularDetectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/RectangularDetectorEditor.h"
#include "GUI/Models/AxesItems.h"
#include "GUI/Models/RectangularDetectorItem.h"
#include "GUI/Models/VectorItem.h"
#include "GUI/Views/InstrumentWidgets/DetectorAlignmentEditor.h"
#include "GUI/Views/InstrumentWidgets/ResolutionFunctionEditor.h"

#include <QDoubleSpinBox>
#include <QFormLayout>
#include <QGridLayout>
#include <QGroupBox>
#include <QSpinBox>
#include <QVBoxLayout>

#include <array>

//==================================================================================================
// RectangularAxisEditor
//==================================================================================================

class RectangularAxisEditor : public QWidget {

public:
    enum Orientation { X_Axis, Y_Axis };
    RectangularAxisEditor(const Orientation& orientation, QWidget* parent = nullptr,
                          Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(BasicAxisItem* item);
    void clearItem();

private slots:
    void onNbinsValueChanged(int value);
    void onLengthValueChanged(double value);

private:
    QSpinBox* m_spinbox_nbins;
    QDoubleSpinBox* m_spinbox_length;
    BasicAxisItem* m_item;
};

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

RectangularAxisEditor::RectangularAxisEditor(const Orientation& orientation, QWidget* parent,
                                             Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    static const std::array<QString, 2> titles = {"X axis", "Y axis"};
    static const std::array<QString, 2> length_labels = {"Width [mm]", "Height [mm]"};

    QVBoxLayout* layout = new QVBoxLayout(this);
    QGroupBox* gbox = new QGroupBox(titles[orientation], this);
    QFormLayout* form = new QFormLayout(gbox);
    layout->addWidget(gbox);

    m_spinbox_nbins = new QSpinBox(gbox);
    m_spinbox_nbins->setRange(1, 65536);
    connect(m_spinbox_nbins, qOverload<int>(&QSpinBox::valueChanged), this,
            &RectangularAxisEditor::onNbinsValueChanged);
    form->addRow("Nbins:", m_spinbox_nbins);

    m_spinbox_length = new QDoubleSpinBox(gbox);
    m_spinbox_length->setRange(std::numeric_limits<double>::lowest(),
                               std::numeric_limits<double>::max());
    m_spinbox_length->setDecimals(3);
    m_spinbox_length->setSingleStep(0.01);
    connect(m_spinbox_length, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &RectangularAxisEditor::onLengthValueChanged);
    form->addRow(length_labels[orientation], m_spinbox_length);
}

void RectangularAxisEditor::setItem(BasicAxisItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        m_spinbox_nbins->setValue(m_item->binCount());
        m_spinbox_length->setValue(m_item->upperBound());
    } else {
        m_spinbox_nbins->setValue(1);
        m_spinbox_length->setValue(0.0);
    }
}

void RectangularAxisEditor::clearItem()
{
    setItem(nullptr);
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void RectangularAxisEditor::onNbinsValueChanged(int value)
{
    if (m_item)
        m_item->setBinCount(value);
}

void RectangularAxisEditor::onLengthValueChanged(double value)
{
    if (m_item)
        m_item->setUpperBound(value);
}

//==================================================================================================
// RectangularDetectorEditor
//==================================================================================================

RectangularDetectorEditor::RectangularDetectorEditor(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f)
{
    QGridLayout* grid = new QGridLayout(this);
    grid->setColumnStretch(0, 1);
    grid->setColumnStretch(1, 1);
    grid->setColumnStretch(2, 1);

    // axes and resolution function editors
    m_xAxisEditor = new RectangularAxisEditor(RectangularAxisEditor::X_Axis, this);
    grid->addWidget(m_xAxisEditor, 1, 0);

    m_yAxisEditor = new RectangularAxisEditor(RectangularAxisEditor::Y_Axis, this);
    grid->addWidget(m_yAxisEditor, 1, 1);

    m_resolutionFunctionEditor =
        new ResolutionFunctionEditor(ResolutionFunctionEditor::Millimeter, this);
    grid->addWidget(m_resolutionFunctionEditor, 1, 2);

    // alignment selector editors
    m_alignmentEditor = new DetectorAlignmentEditor(this);
    grid->addWidget(m_alignmentEditor, 2, 0, 1, 3);
}

void RectangularDetectorEditor::setItem(RectangularDetectorItem* item)
{
    if (item) {
        m_xAxisEditor->setItem(item->xAxisItem());
        m_yAxisEditor->setItem(item->yAxisItem());
        m_resolutionFunctionEditor->setItem(item);
        m_alignmentEditor->setItem(item);
    } else {
        m_xAxisEditor->clearItem();
        m_yAxisEditor->clearItem();
        m_resolutionFunctionEditor->clearItem();
        m_alignmentEditor->clearItem();
    }
}
