//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/ResolutionFunctionEditor.cpp
//! @brief     Implements class ResolutionFunctionEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/ResolutionFunctionEditor.h"
#include "GUI/Models/DetectorItems.h"
#include "GUI/Models/ResolutionFunctionItems.h"

#include <QComboBox>
#include <QDoubleSpinBox>
#include <QFormLayout>
#include <QGroupBox>
#include <QStackedLayout>

namespace {

const QString unitString(const ResolutionFunctionEditor::Unit unit)
{
    switch (unit) {
    case ResolutionFunctionEditor::Degree:
        return "deg";
    case ResolutionFunctionEditor::Millimeter:
        return "mm";
    }
    ASSERT(false);
}

} // namespace

//==================================================================================================
// GaussResolutionFunctionForm
//==================================================================================================

/// form for the parameters of the gauss 2D resolution function
class GaussResolutionFunctionForm : public QWidget {
public:
    GaussResolutionFunctionForm(const ResolutionFunctionEditor::Unit unit,
                                QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(ResolutionFunction2DGaussianItem* item);

private slots:
    void onSigmaXValueChanged(double sigma_x);
    void onSigmaYValueChanged(double sigma_y);

private:
    QDoubleSpinBox* m_spinbox_sigmax;
    QDoubleSpinBox* m_spinbox_sigmay;
    ResolutionFunction2DGaussianItem* m_item;
};

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

GaussResolutionFunctionForm::GaussResolutionFunctionForm(const ResolutionFunctionEditor::Unit unit,
                                                         QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QFormLayout* layout = new QFormLayout(this);
    layout->setMargin(0);

    m_spinbox_sigmax = new QDoubleSpinBox(this);
    m_spinbox_sigmax->setDecimals(3);
    m_spinbox_sigmax->setRange(0, std::numeric_limits<double>::max());
    m_spinbox_sigmax->setSingleStep(0.01);
    m_spinbox_sigmax->setToolTip("Resolution along horizontal axis (in detector units)");
    connect(m_spinbox_sigmax, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &GaussResolutionFunctionForm::onSigmaXValueChanged);
    layout->addRow("sigma X [" + unitString(unit) + "]:", m_spinbox_sigmax);

    m_spinbox_sigmay = new QDoubleSpinBox(this);
    m_spinbox_sigmay->setDecimals(3);
    m_spinbox_sigmay->setRange(0, std::numeric_limits<double>::max());
    m_spinbox_sigmay->setSingleStep(0.01);
    m_spinbox_sigmay->setToolTip("Resolution along vertical axis (in detector units)");
    connect(m_spinbox_sigmay, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &GaussResolutionFunctionForm::onSigmaYValueChanged);
    layout->addRow("sigma Y [" + unitString(unit) + "]:", m_spinbox_sigmay);
}

void GaussResolutionFunctionForm::setItem(ResolutionFunction2DGaussianItem* item)
{
    m_item = item;
    m_spinbox_sigmax->setValue(m_item->sigmaX());
    m_spinbox_sigmay->setValue(m_item->sigmaY());
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void GaussResolutionFunctionForm::onSigmaXValueChanged(double sigma_x)
{
    if (m_item)
        m_item->setSigmaX(sigma_x);
}

void GaussResolutionFunctionForm::onSigmaYValueChanged(double sigma_y)
{
    if (m_item)
        m_item->setSigmaY(sigma_y);
}

//==================================================================================================
// ResolutionFunctionEditor
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

ResolutionFunctionEditor::ResolutionFunctionEditor(Unit unit, QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QHBoxLayout* layout = new QHBoxLayout(this);
    QGroupBox* gbox = new QGroupBox("Resolution function", this);
    QFormLayout* form = new QFormLayout(gbox);
    layout->addWidget(gbox);

    m_combo = new QComboBox(this);
    form->addRow("Type:", m_combo);

    m_stack = new QStackedLayout();
    form->addRow(m_stack);

    m_combo->addItem("None", ResolutionFunctionNoneItem::M_TYPE);
    m_form_blank = new QWidget(this);
    m_stack->addWidget(m_form_blank);

    m_combo->addItem("2D Gaussian", ResolutionFunction2DGaussianItem::M_TYPE);
    m_form_gauss = new GaussResolutionFunctionForm(unit, this);
    m_stack->addWidget(m_form_gauss);

    connect(m_combo, qOverload<int>(&QComboBox::currentIndexChanged), this,
            &ResolutionFunctionEditor::onResolutionFunctionSelected);
}

void ResolutionFunctionEditor::setItem(DetectorItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        int idx = m_combo->findData(m_item->resolutionFunction()->modelType());
        ASSERT(idx != -1);
        if (idx != m_combo->currentIndex())
            m_combo->setCurrentIndex(idx);
        else
            onResolutionFunctionSelected(idx); // trigger update manually
    }
}

void ResolutionFunctionEditor::clearItem()
{
    setItem(nullptr);
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void ResolutionFunctionEditor::onResolutionFunctionSelected(int index)
{
    if (m_item && index != -1) {
        m_item->setResolutionFunctionType(m_combo->currentData().toString());
        if (m_combo->currentData() == ResolutionFunctionNoneItem::M_TYPE)
            m_stack->setCurrentWidget(m_form_blank);
        else {
            m_form_gauss->setItem(
                dynamic_cast<ResolutionFunction2DGaussianItem*>(m_item->resolutionFunction()));
            m_stack->setCurrentWidget(m_form_gauss);
        }
    }
}
