//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/SphericalAxisEditor.cpp
//! @brief     Implement class SphericalAxisEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/SphericalAxisEditor.h"
#include "GUI/Models/AxesItems.h"

#include <QDoubleSpinBox>
#include <QFormLayout>
#include <QGroupBox>
#include <QSpinBox>


//==================================================================================================
// SphericalAxisEditor
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

SphericalAxisEditor::SphericalAxisEditor(const QString& title, QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QVBoxLayout* layout = new QVBoxLayout(this);
    QGroupBox* gbox = new QGroupBox(title, this);
    QFormLayout* form = new QFormLayout(gbox);
    layout->addWidget(gbox);

    m_spinbox_nbins = new QSpinBox(gbox);
    m_spinbox_nbins->setRange(1, 65536);
    connect(m_spinbox_nbins, qOverload<int>(&QSpinBox::valueChanged), this,
            &SphericalAxisEditor::onNbinsValueChanged);
    form->addRow("Nbins:", m_spinbox_nbins);

    m_spinbox_minimum = new QDoubleSpinBox(gbox);
    m_spinbox_minimum->setRange(std::numeric_limits<double>::lowest(),
                                std::numeric_limits<double>::max());
    m_spinbox_minimum->setDecimals(3);
    m_spinbox_minimum->setSingleStep(0.01);
    connect(m_spinbox_minimum, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &SphericalAxisEditor::onMinimumValueChanged);
    form->addRow("Minimum [deg]:", m_spinbox_minimum);

    m_spinbox_maximum = new QDoubleSpinBox(gbox);
    m_spinbox_maximum->setRange(std::numeric_limits<double>::lowest(),
                                std::numeric_limits<double>::max());
    m_spinbox_maximum->setDecimals(3);
    m_spinbox_maximum->setSingleStep(0.01);
    connect(m_spinbox_maximum, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &SphericalAxisEditor::onMaximumValueChanged);
    form->addRow("Maximum [deg]:", m_spinbox_maximum);
}

void SphericalAxisEditor::setItem(BasicAxisItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        m_spinbox_nbins->setValue(m_item->binCount());
        m_spinbox_minimum->setValue(m_item->lowerBound());
        m_spinbox_maximum->setValue(m_item->upperBound());
    } else {
        m_spinbox_nbins->setValue(1);
        m_spinbox_minimum->setValue(0.0);
        m_spinbox_maximum->setValue(0.0);
    }
}

void SphericalAxisEditor::clearItem()
{
    setItem(nullptr);
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void SphericalAxisEditor::onNbinsValueChanged(int value)
{
    if (m_item)
        m_item->setBinCount(value);
}

void SphericalAxisEditor::onMinimumValueChanged(double value)
{
    if (m_item) {
        m_item->setLowerBound(value);
        if (m_item->upperBound() < value)
            m_spinbox_maximum->setValue(value);
    }
}

void SphericalAxisEditor::onMaximumValueChanged(double value)
{
    if (m_item) {
        m_item->setUpperBound(value);
        if (m_item->lowerBound() > value)
            m_spinbox_minimum->setValue(value);
    }
}
