//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/VectorEditor.cpp
//! @brief     Implements class VectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/VectorEditor.h"
#include "GUI/Models/VectorItem.h"

#include <QDoubleSpinBox>
#include <QFormLayout>
#include <QGroupBox>

#include <limits>

//==================================================================================================
// VectorEditor
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

VectorEditor::VectorEditor(const QString& title, const QString& tooltip, QWidget* parent,
                           Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QVBoxLayout* layout = new QVBoxLayout(this);

    QGroupBox* gbox = new QGroupBox(title, this);
    layout->addWidget(gbox);
    QFormLayout* form = new QFormLayout(gbox);

    m_spinbox_x = new QDoubleSpinBox(gbox);
    m_spinbox_x->setToolTip(tooltip.arg("x coordinate"));
    m_spinbox_x->setDecimals(4);
    m_spinbox_x->setRange(std::numeric_limits<double>::lowest(),
                          std::numeric_limits<double>::max());
    form->addRow("X:", m_spinbox_x);

    m_spinbox_y = new QDoubleSpinBox(gbox);
    m_spinbox_y->setToolTip(tooltip.arg("y coordinate"));
    m_spinbox_y->setDecimals(4);
    m_spinbox_y->setRange(std::numeric_limits<double>::lowest(),
                          std::numeric_limits<double>::max());
    form->addRow("Y:", m_spinbox_y);

    m_spinbox_z = new QDoubleSpinBox(gbox);
    m_spinbox_z->setToolTip(tooltip.arg("z coordinate"));
    m_spinbox_z->setDecimals(4);
    m_spinbox_z->setRange(std::numeric_limits<double>::lowest(),
                          std::numeric_limits<double>::max());
    form->addRow("Z:", m_spinbox_z);

    connect(m_spinbox_x, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &VectorEditor::onXValueChanged);
    connect(m_spinbox_y, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &VectorEditor::onYValueChanged);
    connect(m_spinbox_z, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &VectorEditor::onZValueChanged);
}

void VectorEditor::setItem(VectorItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        m_spinbox_x->setValue(m_item->x());
        m_spinbox_y->setValue(m_item->y());
        m_spinbox_z->setValue(m_item->z());
    } else {
        m_spinbox_x->setValue(0);
        m_spinbox_y->setValue(0);
        m_spinbox_z->setValue(0);
    }
}

VectorItem* VectorEditor::item() const
{
    return m_item;
}

void VectorEditor::clearItem()
{
    setItem(nullptr);
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void VectorEditor::onXValueChanged(double x)
{
    if (m_item) {
        m_item->setX(x);
        emit vectorChanged();
    }
}

void VectorEditor::onYValueChanged(double y)
{
    if (m_item) {
        m_item->setY(y);
        emit vectorChanged();
    }
}

void VectorEditor::onZValueChanged(double z)
{
    if (m_item) {
        m_item->setZ(z);
        emit vectorChanged();
    }
}
