//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/mainwindow/projectdocument.h
//! @brief     Defines class ProjectDocument
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MAINWINDOW_PROJECTDOCUMENT_H
#define BORNAGAIN_GUI_MAINWINDOW_PROJECTDOCUMENT_H

#include "GUI/Models/ApplicationModels.h"
#include <QObject>

class QIODevice;
class ApplicationModels;
class MessageService;
class OutputDataIOService;
class InstrumentModel;
class SampleModel;
class MaterialModel;
class RealDataModel;
class JobModel;
class LinkInstrumentManager;

//! Project document class handles all data related to the opened project
//! (sample, jobModel, project specific windows settings)
//!
//! e.g. if project file is /home/users/development/Untitled/Untitled.pro
//! projectName()     - 'Untitled'
//! projectDir()      - '/home/users/development/Untitled
//! projectFileName() - '/home/users/development/Untitled/Untitled.pro'
class ProjectDocument : public QObject {
    Q_OBJECT

public:
    enum class ReadResult { ok, warning, error };

    enum Functionality {
        None = 0x0,
        Gisas = 0x1,
        OffSpecular = 0x2,
        Specular = 0x4,
        DepthProbe = 0x8,
        All = Gisas | OffSpecular | Specular | DepthProbe
    };
    Q_DECLARE_FLAGS(Functionalities, Functionality)

    ProjectDocument(const QString& projectFileName = "");

    QString projectName() const;
    void setProjectName(const QString& text);

    QString projectDir() const;
    void setProjectDir(const QString& text);

    QString projectFileName() const;
    void setProjectFileName(const QString& text);

    static QString projectFileExtension();

    ApplicationModels* applicationModels();
    InstrumentModel* instrumentModel() const;
    SampleModel* sampleModel() const;
    MaterialModel* materialModel() const;
    RealDataModel* realDataModel() const;
    JobModel* jobModel() const;
    LinkInstrumentManager* linkInstrumentManager();

    void save(const QString& project_file_name, bool autoSave = false);

    void saveProjectFile(const QString& project_file_name, bool autoSave = false);
    void saveProjectData(const QString& project_file_name);

    ReadResult loadProjectFile(const QString& project_file_name, MessageService& messageService);

    bool hasValidNameAndPath();

    bool isModified();
    void setModified(bool flag = true);

    bool hasData() const;

    QString documentVersion() const;

    bool singleInstrumentMode() const;
    void setSingleInstrumentMode(bool b);

    Functionalities functionalities() const;
    void setFunctionalities(const Functionalities& f);

signals:
    void modified();

private:
    ReadResult readProject(QIODevice* device, MessageService& messageService);
    void writeTo(QIODevice* device);

    void disconnectModels();
    void connectModels();
    void onModelChanged();

    QString m_project_dir;
    QString m_project_name;
    ApplicationModels m_applicationModels;
    bool m_modified;
    QString m_currentVersion;
    OutputDataIOService* m_dataService;
    LinkInstrumentManager* m_linkManager;
    bool m_singleInstrumentMode;
    Functionalities m_functionalities;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(ProjectDocument::Functionalities)

#endif // BORNAGAIN_GUI_MAINWINDOW_PROJECTDOCUMENT_H
