#include "Param/Node/INode.h"
#include "Param/Node/NodeUtils.h"
#include "Fit/Param/RealLimits.h"
#include "Tests/GTestWrapper/google_test.h"
#include <memory>
#include <stdexcept>

namespace {
const std::string test_class_name = "TestClass";
const std::string another_test_class_name = "AnotherTestClass";
const double test_par1_value(1.0);
} // namespace

class INodeTest : public ::testing::Test {
public:
    class TestClass : public INode {
    public:
        std::string className() const final { return "TestClass"; }

        TestClass(const std::string& name = test_class_name, double value = test_par1_value)
            : m_parameter1(value)
        {
        }

        virtual ~TestClass()
        {
            for (auto child : m_nodes)
                delete child;
        }
        void accept(INodeVisitor* visitor) const override { visitor->visit(this); }

        void appendChild(INode* node)
        {
            m_nodes.push_back(node);
            registerChild(node);
        }

        std::vector<const INode*> getChildren() const override
        {
            return {m_nodes.begin(), m_nodes.end()};
        }

        std::vector<INode*> m_nodes;
        double m_parameter1;

        friend class INodeTest_fullQualifiedParameters_Test;
        friend class INodeTest_createChildParameterTree_Test;
    };
};

TEST_F(INodeTest, initialState)
{
    INodeTest::TestClass node;
    EXPECT_EQ(node.getChildren().size(), 0u);
    EXPECT_EQ(node.parent(), nullptr);
}

TEST_F(INodeTest, appendChild)
{
    INodeTest::TestClass node;

    INodeTest::TestClass* child0 = new INodeTest::TestClass();

    // inserting single child
    node.appendChild(child0);

    // accessing vector of children
    auto children = node.getChildren();
    EXPECT_EQ(children.size(), 1u);
    EXPECT_TRUE(std::find(children.begin(), children.end(), child0) != children.end());
}

//! Checks change of parentship on insert/detach.

TEST_F(INodeTest, parentship)
{
    INodeTest::TestClass node;
    EXPECT_EQ(node.parent(), nullptr);

    INodeTest::TestClass* child = new INodeTest::TestClass();
    node.appendChild(child);
    EXPECT_EQ(child->parent(), &node);
}

TEST_F(INodeTest, checkLimits)
{
    // this checks only the function if RealLimits.check() works in principle.
    // More tests regarding RealLimits should be done in dedicated RealLimits unit tests.
    EXPECT_NO_THROW(RealLimits::nonnegative().check("A", 0));
    EXPECT_THROW(RealLimits::positive().check("A", 0), std::runtime_error);
}
